package cn.com.infosec.jce.provider;

import cn.com.infosec.jce.X509LDAPCertStoreParameters;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CRL;
import java.security.cert.CRLSelector;
import java.security.cert.CertSelector;
import java.security.cert.CertStoreException;
import java.security.cert.CertStoreParameters;
import java.security.cert.CertStoreSpi;
import java.security.cert.CertificateFactory;
import java.security.cert.X509CRLSelector;
import java.security.cert.X509CertSelector;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.security.auth.x500.X500Principal;

public class X509LDAPCertStoreSpi extends CertStoreSpi {
   private static String LDAP_PROVIDER;
   private static String REFERRALS_IGNORE;
   private static final String SEARCH_SECURITY_LEVEL = "none";
   private static final String URL_CONTEXT_PREFIX = "com.sun.jndi.url";
   private X509LDAPCertStoreParameters params;

   public X509LDAPCertStoreSpi(CertStoreParameters params) {
      super(params);
      if (params instanceof X509LDAPCertStoreParameters) {
         this.params = (X509LDAPCertStoreParameters)params;
      } else {
         throw new InvalidAlgorithmParameterException("cn.com.infosec.jce.provider.LDAPCertStoreSpi: parameter must be a LDAPCertStoreParameters object\n" + params.toString());
      }
   }

   private DirContext connectLDAP() {
      Properties var1;
      Properties var10000 = var1 = new Properties;
      var1.<init>();
      var1.setProperty("java.naming.factory.initial", LDAP_PROVIDER);
      var1.setProperty("java.naming.batchsize", "0");
      var10000.setProperty("java.naming.provider.url", this.params.getLdapURL());
      var10000.setProperty("java.naming.factory.url.pkgs", "com.sun.jndi.url");
      var10000.setProperty("java.naming.referral", REFERRALS_IGNORE);
      var10000.setProperty("java.naming.security.authentication", "none");
      return new InitialDirContext(var1);
   }

   private String parseDN(String subject, String subjectAttributeName) {
      String var3;
      int subject1;
      if ((subject1 = (var3 = subject.substring(subject.toLowerCase().indexOf(subjectAttributeName.toLowerCase()) + subjectAttributeName.length())).indexOf(44)) == -1) {
         subject1 = var3.length();
      }

      while(var3.charAt(subject1 - 1) == '\\') {
         if ((subject1 = var3.indexOf(44, subject1 + 1)) == -1) {
            subject1 = var3.length();
         }
      }

      String var10000 = var3.substring(0, subject1);
      if ((var3 = var10000.substring(var10000.indexOf(61) + 1)).charAt(0) == ' ') {
         var3 = var3.substring(1);
      }

      if (var3.startsWith("\"")) {
         var3 = var3.substring(1);
      }

      if (var3.endsWith("\"")) {
         var3 = var3.substring(0, var3.length() - 1);
      }

      return var3;
   }

   private Set certSubjectSerialSearch(X509CertSelector xselector, String[] attrs, String attrName, String subjectAttributeName) {
      X509CertSelector var10000 = xselector;
      HashSet var5;
      var5 = new HashSet.<init>();

      IOException var25;
      label134: {
         boolean var10001;
         byte[] var26;
         try {
            var26 = var10000.getSubjectAsBytes();
         } catch (IOException var22) {
            var25 = var22;
            var10001 = false;
            break label134;
         }

         String var27;
         X509Certificate var28;
         if (var26 == null) {
            try {
               var27 = xselector.getSubjectAsString();
            } catch (IOException var21) {
               var25 = var21;
               var10001 = false;
               break label134;
            }

            if (var27 == null) {
               try {
                  var28 = xselector.getCertificate();
               } catch (IOException var20) {
                  var25 = var20;
                  var10001 = false;
                  break label134;
               }

               if (var28 == null) {
                  try {
                     var5.addAll(this.search(attrName, "*", attrs));
                     return var5;
                  } catch (IOException var7) {
                     var25 = var7;
                     var10001 = false;
                     break label134;
                  }
               }
            }
         }

         var10000 = xselector;
         String var6 = null;

         try {
            var28 = var10000.getCertificate();
         } catch (IOException var19) {
            var25 = var19;
            var10001 = false;
            break label134;
         }

         String xselector1;
         if (var28 != null) {
            String var30;
            try {
               var10000 = xselector;
               var30 = xselector.getCertificate().getSubjectX500Principal().getName("RFC1779");
            } catch (IOException var18) {
               var25 = var18;
               var10001 = false;
               break label134;
            }

            xselector1 = var30;

            try {
               var27 = var10000.getCertificate().getSerialNumber().toString();
            } catch (IOException var17) {
               var25 = var17;
               var10001 = false;
               break label134;
            }

            var6 = var27;
         } else {
            try {
               var26 = xselector.getSubjectAsBytes();
            } catch (IOException var16) {
               var25 = var16;
               var10001 = false;
               break label134;
            }

            if (var26 != null) {
               try {
                  var27 = (new X500Principal(xselector.getSubjectAsBytes())).getName("RFC1779");
               } catch (IOException var15) {
                  var25 = var15;
                  var10001 = false;
                  break label134;
               }
            } else {
               try {
                  var27 = xselector.getSubjectAsString();
               } catch (IOException var14) {
                  var25 = var14;
                  var10001 = false;
                  break label134;
               }
            }

            xselector1 = var27;
         }

         X509LDAPCertStoreSpi var10002;
         String var10003;
         String var10004;
         HashSet var33;
         try {
            var27 = var6;
            var33 = var5;
            var10002 = this;
            var10003 = attrName;
            var10004 = this.parseDN(xselector1, subjectAttributeName);
         } catch (IOException var13) {
            var25 = var13;
            var10001 = false;
            break label134;
         }

         xselector1 = var10004;

         StringBuilder var32;
         try {
            var32 = new StringBuilder;
         } catch (IOException var12) {
            var25 = var12;
            var10001 = false;
            break label134;
         }

         StringBuilder var10005 = var32;

         try {
            var10005.<init>("*");
            var33.addAll(var10002.search(var10003, var32.append(xselector1).append("*").toString(), attrs));
         } catch (IOException var11) {
            var25 = var11;
            var10001 = false;
            break label134;
         }

         if (var27 == null) {
            return var5;
         }

         try {
            var27 = this.params.getSearchForSerialNumberIn();
         } catch (IOException var10) {
            var25 = var10;
            var10001 = false;
            break label134;
         }

         if (var27 == null) {
            return var5;
         }

         String var29;
         StringBuilder var31;
         X509LDAPCertStoreSpi var34;
         HashSet var35;
         try {
            var35 = var5;
            var34 = this;
            var29 = this.params.getSearchForSerialNumberIn();
            var31 = new StringBuilder;
         } catch (IOException var9) {
            var25 = var9;
            var10001 = false;
            break label134;
         }

         var32 = var31;

         try {
            var32.<init>("*");
            var35.addAll(var34.search(var29, var31.append(var6).append("*").toString(), attrs));
            return var5;
         } catch (IOException var8) {
            var25 = var8;
            var10001 = false;
         }
      }

      IOException var23 = var25;
      throw new CertStoreException("exception processing selector: " + var23);
   }

   private Set getEndCertificates(X509CertSelector xselector) {
      X509LDAPCertStoreSpi var10000 = this;
      X509CertSelector var10001 = xselector;
      X509LDAPCertStoreSpi var10002 = this;
      String[] xselector1;
      (xselector1 = new String[1])[0] = this.params.getUserCertificateAttribute();
      String var3 = this.params.getLdapUserCertificateAttributeName();
      String var2 = var10002.params.getUserCertificateSubjectAttributeName();
      return var10000.certSubjectSerialSearch(var10001, xselector1, var3, var2);
   }

   private Set getCACertificates(X509CertSelector xselector) {
      X509CertSelector var10001 = xselector;
      String[] xselector1;
      (xselector1 = new String[1])[0] = this.params.getCACertificateAttribute();
      String var2 = this.params.getLdapCACertificateAttributeName();
      String var3 = this.params.getCACertificateSubjectAttributeName();
      Set var5;
      if ((var5 = this.certSubjectSerialSearch(var10001, xselector1, var2, var3)).isEmpty()) {
         var5.addAll(this.search((String)null, "*", xselector1));
      }

      return var5;
   }

   private Set getCrossCertificates(X509CertSelector xselector) {
      X509CertSelector var10001 = xselector;
      String[] xselector1;
      (xselector1 = new String[1])[0] = this.params.getCrossCertificateAttribute();
      String var2 = this.params.getLdapCrossCertificateAttributeName();
      String var3 = this.params.getCrossCertificateSubjectAttributeName();
      Set var5;
      if ((var5 = this.certSubjectSerialSearch(var10001, xselector1, var2, var3)).isEmpty()) {
         var5.addAll(this.search((String)null, "*", xselector1));
      }

      return var5;
   }

   private Set search(String param1, String param2, String[] param3) {
      // $FF: Couldn't be decompiled
   }

   public Collection engineGetCertificates(CertSelector param1) {
      // $FF: Couldn't be decompiled
   }

   public Collection engineGetCRLs(CRLSelector selector) {
      String[] var2;
      (var2 = new String[1])[0] = this.params.getCertificateRevocationListAttribute();
      if (!(selector instanceof X509CRLSelector)) {
         throw new CertStoreException("selector is not a X509CRLSelector");
      } else {
         X509CRLSelector selector1;
         X509CRLSelector var10000 = selector1 = (X509CRLSelector)selector;
         HashSet var3;
         var3 = new HashSet.<init>();
         String var4 = this.params.getLdapCertificateRevocationListAttributeName();
         HashSet var5;
         var5 = new HashSet.<init>();
         String var18;
         if (var10000.getIssuerNames() != null) {
            for(Iterator var6 = selector1.getIssuerNames().iterator(); var6.hasNext(); var5.addAll(this.search(var4, "*" + var18 + "*", var2))) {
               Object var7;
               if ((var7 = var6.next()) instanceof String) {
                  Object var10001 = var7;
                  var18 = this.params.getCertificateRevocationListIssuerAttributeName();
                  var18 = this.parseDN((String)var10001, var18);
               } else {
                  var18 = this.parseDN((new X500Principal((byte[])var7)).getName("RFC1779"), this.params.getCertificateRevocationListIssuerAttributeName());
               }
            }
         } else {
            var5.addAll(this.search(var4, "*", var2));
         }

         var5.addAll(this.search((String)null, "*", var2));
         Iterator var13 = var5.iterator();

         Exception var19;
         label75: {
            CertificateFactory var20;
            boolean var21;
            try {
               var20 = CertificateFactory.getInstance("X.509", "INFOSEC");
            } catch (Exception var12) {
               var19 = var12;
               var21 = false;
               break label75;
            }

            CertificateFactory var16 = var20;

            while(true) {
               boolean var22;
               try {
                  var22 = var13.hasNext();
               } catch (Exception var11) {
                  var19 = var11;
                  var21 = false;
                  break;
               }

               if (!var22) {
                  return var3;
               }

               CRL var23;
               try {
                  var10000 = selector1;
                  var23 = var16.generateCRL(new ByteArrayInputStream((byte[])var13.next()));
               } catch (Exception var10) {
                  var19 = var10;
                  var21 = false;
                  break;
               }

               CRL var17 = var23;

               try {
                  var22 = var10000.match(var23);
               } catch (Exception var9) {
                  var19 = var9;
                  var21 = false;
                  break;
               }

               if (var22) {
                  try {
                     var3.add(var17);
                  } catch (Exception var8) {
                     var19 = var8;
                     var21 = false;
                     break;
                  }
               }
            }
         }

         Exception var14 = var19;
         throw new CertStoreException("CRL cannot be constructed from LDAP result " + var14);
      }
   }
}
